@@ -1,6 +1,6 @@
 ;;; speechd.el --- Library for accessing Speech Dispatcher
 
-;; Copyright (C) 2003, 2004 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -18,7 +18,7 @@
 ;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 ;;; Commentary:
 
@@ -41,13 +41,12 @@
 
 (eval-when-compile (require 'cl))
 
+(require 'speechd-common)
+
 
 ;;; User variables
 
 
-(defgroup speechd-el ()
-  "Speechd-el speech output system.")
-
 (defgroup speechd ()
   "SSIP interface."
   :group 'speechd-el)
@@ -95,26 +94,6 @@ variable is nil, Emacs talks to SSIP directly."
                            (or (eq value nil)
                                (assoc value speechd-voices))))))
 
-(defcustom speechd-default-text-priority 'text
-  "Default Speech Dispatcher priority of sent texts."
-  :type 'speechd-priority-tag
-  :group 'speechd)
-
-(defcustom speechd-default-sound-priority 'message
-  "Default Speech Dispatcher priority of sent sound icons."
-  :type 'speechd-priority-tag
-  :group 'speechd)
-
-(defcustom speechd-default-char-priority 'notification
-  "Default Speech Dispatcher priority of sent single letters."
-  :type 'speechd-priority-tag
-  :group 'speechd)
-
-(defcustom speechd-default-key-priority 'notification
-  "Default Speech Dispatcher priority of sent symbols of keys."
-  :type 'speechd-priority-tag
-  :group 'speechd)
-
 
 (defmacro speechd--generate-customization-options (var)
   (let ((value (symbol-value var)))
@@ -233,18 +212,6 @@ current voice."
 ;;; External variables
 
 
-(defvar speechd-client-name "default"
-  "String defining current client name.
-This variable's value defines which connection is used when communicating with
-Speech Dispatcher, each connection has its own client name.  Usually, you
-select the proper client (connection) by assigning a value to this variable
-locally through `let'.")
-
-(defvar speechd-language nil
-  "If non-nil, it is an RFC 1766 language code, as a string.
-If text is read and this variable is non-nil, the text is read in the given
-language.")
-
 (defvar speechd-spell nil
   "If non-nil, any spoken text is spelled.")
 
@@ -426,13 +393,6 @@ language.")
   `(speechd--with-connection-parameters (speechd--voice-parameters ,voice)
      ,@body))
 
-(defun speechd-language (string language)
-  "Put language property LANGUAGE on whole STRING.
-Language should be a string recognizable by Speech Dispatcher as a language
-code."
-  (put-text-property 0 (length string) 'language language string)
-  string)
-
 (defun speechd--current-language ()
   speechd-language)
 
@@ -456,12 +416,11 @@ code."
 (defun speechd--call-spdsend (args &optional input)
   (with-temp-buffer
     (if input
-        (progn
+        (with-speechd-coding-protection
           (insert input)
           (let ((default-process-coding-system
                   (cons speechd--coding-system speechd--coding-system))
-                (process-coding-system-alist nil)
-                (last-coding-system-used nil))
+                (process-coding-system-alist nil))
             (apply #'call-process-region (point-min) (point-max)
                    speechd-spdsend t t nil args)))
       (apply #'call-process speechd-spdsend nil t nil args))
@@ -508,25 +467,26 @@ code."
 (defun speechd--send-connection (connection command)
   (let ((process (speechd--connection-process connection)))
     (when process
-      (condition-case _
-          (if (speechd--use-spdsend)
-              (speechd--call-spdsend (list "--send" process) command)
-            (process-send-string
-             process
-             (if speechd--advanced-apo
-                 command
-               (encode-coding-string command speechd--coding-system)))
-            (when speechd--advanced-apo
-              (while (not (string-match
-                           speechd--end-regexp
-                           (speechd--connection-process-output connection)))
-                (unless (accept-process-output process speechd-timeout nil 1)
-                  (error "Timeout in communication with Speech Dispatcher"))))
-            (prog1 (speechd--connection-process-output connection)
-              (setf (speechd--connection-process-output connection) "")))
-        (error
-         (speechd--permanent-connection-failure connection)
-         (error "Error in communication with Speech Dispatcher"))))))
+      (with-speechd-coding-protection
+        (condition-case _
+            (if (speechd--use-spdsend)
+                (speechd--call-spdsend (list "--send" process) command)
+              (process-send-string
+               process
+               (if speechd--advanced-apo
+                   command
+                 (encode-coding-string command speechd--coding-system)))
+              (when speechd--advanced-apo
+                (while (not (string-match
+                             speechd--end-regexp
+                             (speechd--connection-process-output connection)))
+                  (unless (accept-process-output process speechd-timeout nil 1)
+                    (error "Timeout in communication with Speech Dispatcher"))))
+              (prog1 (speechd--connection-process-output connection)
+                (setf (speechd--connection-process-output connection) "")))
+          (error
+           (speechd--permanent-connection-failure connection)
+           (error "Error in communication with Speech Dispatcher")))))))
   
 ;;;###autoload
 (defun* speechd-open (&optional host port &key quiet force-reopen)
@@ -690,12 +650,12 @@ Return the opened connection on success, nil otherwise."
   (unless (listp command)
     (setq command (list command)))
   (speechd--with-current-connection
-    (setf (speechd--connection-last-command connection) command)
     (when (or (not (speechd--connection-in-block connection))
               (speechd--block-command-p command))
+      (setf (speechd--connection-last-command connection) command)
       (speechd--send-request
-        (make-speechd--request
-         :string (concat (mapconcat #'identity command " ") speechd--eol))))))
+       (make-speechd--request
+        :string (concat (mapconcat #'identity command " ") speechd--eol))))))
 
 (defun speechd--send-text (text)
   (when (first (or (speechd--send-command "SPEAK") '(t)))
@@ -780,7 +740,15 @@ Return the opened connection on success, nil otherwise."
         (when (first answer)
           (setf (speechd--connection-parameters connection)
                 (plist-put (speechd--connection-parameters connection)
-                           parameter value)))))))
+                           parameter value))))
+      ;; Speech Dispatcher bug work-around
+      (when (eq parameter 'language)
+        (let ((output-module (plist-get
+                              (speechd--connection-parameters connection)
+                              'output-module)))
+          (when output-module
+            (speechd--set-connection-parameter 'output-module
+                                               output-module)))))))
 
 (defun speechd--set-parameter (parameter value &optional all)
   (if all
@@ -804,8 +772,7 @@ Return the opened connection on success, nil otherwise."
   "Set language of the current client connection to LANGUAGE.
 Language must be an RFC 1766 language code, as a string."
   (interactive (list (read-string "Language: ")))
-  (speechd--set-parameter 'language language)
-  (setq speechd-language language))
+  (speechd--set-parameter 'language language))
 
 (defmacro speechd--generate-set-command (parameter prompt argdesc)
   (let* ((prompt* (concat prompt ": "))
@@ -909,31 +876,33 @@ If called with a prefix argument, set it for all connections."
 ;;; Blocks
 
 
-(defmacro speechd-block (parameters &rest body)
+(defun speechd-block (function &optional parameters)
+  "Call FUNCTION inside an SSIP block.
+FUNCTION is called without any arguments."
+  (speechd--with-current-connection
+    (let ((%block-connection connection)
+          (nested (and connection (speechd--connection-in-block connection))))
+      (speechd--with-connection-parameters parameters
+        (let ((speechd-client-name (speechd--connection-name connection)))
+          (unless nested
+            (speechd--send-command '("BLOCK BEGIN"))
+            (when connection
+              (setf (speechd--connection-in-block connection) t)))
+          (unwind-protect (funcall function)
+            (unless nested
+              (let ((connection %block-connection))
+                (when connection
+                  (setf (speechd--connection-in-block connection) nil)
+                  (speechd--send-command '("BLOCK END")))))))))))
+
+(defmacro speechd-block* (parameters &rest body)
   "Set PARAMETERS and enclose BODY by an SSIP block.
 Before invoking BODY, the BLOCK BEGIN command is sent, and the BLOCK END
 command is sent afterwards.
 PARAMETERS is a property list defining parameters to be set before sending the
 BLOCK BEGIN command.  The property-value pairs correspond to the arguments of
 the `speechd--set-parameter' function."
-  `(speechd--with-current-connection
-     (speechd--with-connection-parameters ,parameters
-       (if (and connection (speechd--connection-in-block connection))
-           (progn ,@body)
-         (let ((block-connection connection))
-           (speechd--send-command '("BLOCK BEGIN"))
-           (unwind-protect
-               (progn
-                 (speechd--with-current-connection
-                  (when connection
-                    (setf (speechd--connection-in-block connection) t)))
-                 ,@body)
-             (let ((connection block-connection))
-               (when connection
-                 (setf (speechd--connection-in-block connection) nil)
-                 (let ((speechd-client-name
-                        (speechd--connection-name connection)))
-                   (speechd--send-command '("BLOCK END")))))))))))
+  `(speechd-block (lambda () ,@body) ,parameters))
 
 
 ;;; Speaking functions
@@ -951,7 +920,6 @@ is empty."
   (interactive "sText: ")
   (when (or say-if-empty
             (not (string= text "")))
-    (speechd--set-parameter 'message-priority priority)
     (flet ((properties (point)
              (let ((voice (cdr (assq (get-text-property point 'face text)
                                      speechd-face-voices)))
@@ -960,27 +928,26 @@ is empty."
                        (when language (list 'language language))
                        (when (and voice (symbolp voice))
                          (speechd--voice-parameters voice))))))
-      (speechd-block `(language ,(speechd--current-language)
-                       spelling-mode ,speechd-spell)
-        (let* ((beg 0)
-               (new-properties (properties beg)))
-          (while beg
-            (let* ((properties new-properties)
-                   (change-point beg)
-                   (end (progn
-                          (while (and
-                                  (setq change-point (next-property-change
-                                                      change-point text))
-                                  (equal properties
-                                         (setq new-properties
-                                               (properties change-point)))))
-                          change-point))
-                   (substring (substring text beg end)))
-              (if properties
-                  (speechd--with-connection-parameters properties
-                    (speechd--send-text substring))
-                (speechd--send-text substring))
-              (setq beg end))))))))
+      (let* ((beg 0)
+             (new-properties (properties beg)))
+        (while beg
+          (let* ((properties new-properties)
+                 (change-point beg)
+                 (end (progn
+                        (while (and
+                                (setq change-point (next-property-change
+                                                    change-point text))
+                                (equal properties
+                                       (setq new-properties
+                                             (properties change-point)))))
+                        change-point))
+                 (substring (substring text beg end)))
+            (speechd-block* `(message-priority ,priority
+                              language ,(speechd--current-language)
+                              spelling-mode ,speechd-spell
+                              ,@properties)
+              (speechd--send-text substring))
+            (setq beg end)))))))
 
 (defun* speechd-say-sound (name &key (priority speechd-default-sound-priority))
   "Ask speechd to play an auditory icon.
